%function improvement(Img,Colo,JPG?);  

%improvement implement the algorithm before the JPEG compression.
%
%	Img defines the Image to be compressed.
%	Colo defines the color scale of the Img:
%		RGB: 		  Colo=1
%		Grayscale: 	  Colo=2
%	JPG? defines if the Img is already at format JPEG or not.
%		JPEG: JPG?=1
%		else: JPG?=0
%
%	uses functions:
%		neighbors
%		gather
%



function improvement(Img,Colo,JPG?);  

% close all of the windows that are opened.
close all;

global DIFF_PARAM;		% a threshold value ,the number of gray levels that separate between objects.
				% this parameter can change between different Imgs.
                        
% These matrices were defined as global, in order to make the program faster.
% Some different functions work on that Imgs and using the Imgs as parameters 
% to the function makes it much slower.
global ORIGINAL_PIC;		% holds the original Img which is in gray levels
global MORPH_PIC;		% holds the Img after the morphological operators
global CLUSTER;			% a matrice that represents the Img. all the pixels that are
				% part of the same object, will finally have the same value in 
                                % this matrice.
                        
DIFF_PARAM=15; 			% the default value that works for all the of the pictures that were examined.

pict=imread(Img);

if Colo == 1
   gray_pic=256*rgb2gray(im2double(pict));  	   % for RGB pictures
end;

if Colo == 2
   gray_pic=256*mat2gray(im2double(pict),[0,1]);   % for BW Imgs converting picture into Intencity matrice 
end;


ORIGINAL_PIC=gray_pic;

colormap('gray');
imagesc(ORIGINAL_PIC);
title('The original image');
pixval on;

SE = [0 1 0;1 1 1;0 1 0]; 		% structure element
AP = [2 2];				% active point

% morphology opening (erosion + dilation)
MORPH_PIC = imerode(ORIGINAL_PIC,SE);
MORPH_PIC = imdilate(MORPH_PIC,SE);

figure;
colormap('gray');
imagesc(MORPH_PIC);
pixval on;
title('The image after morphology');

[no_rows,no_cols]=size(ORIGINAL_PIC);  % get the no. of rows and columns
													% of the picture's matrice
                           
index=1;                               % index of the object in the cluster image
                           
CLUSTER=zeros(no_rows,no_cols);	       % initializing CLUSTER as a zero matrice 
													% at the size of the original Img
                           
                                       
% labeling the Img using the Region Growing Algo.                                       
for r=1:no_rows
   for c=1:no_cols
      if (r==1)&(c==1) 
           CLUSTER(r,c)=index;         % the first object
        else 
                                       % checking the values of the upper pixel and the pixel to the left
           state=neighbor(r,c);
           
          switch(state)
          case 0,		       % a new object was found	
             index=index+1;					
             CLUSTER(r,c)=index;	
          
          case 1,		       % the current pixel is similiar to the upper one
             CLUSTER(r,c)=CLUSTER(r-1,c);
         		   
          case 2,		       % the current pixel is similiar to the left one
             CLUSTER(r,c)=CLUSTER(r,c-1);
             
          case 3,		       % the current pixel is similiar to the upper one
             			       % and to the left one. the objects should have
             			       % the same index, the index of the upper pixel.
             CLUSTER(r,c)=CLUSTER(r-1,c);
             update_index_corner(r,c);	
         otherwise
            break;
         end;
      end;
   end;
end;


figure;
colormap('gray');
imagesc(CLUSTER);
pixval on;
title('The CLUSTER matrice');

% run through the indexes that were found.
% for each index, get all of it's apperances in CLUSTER
% if there are none, go to the next index. else,
% build a matrix to represent the object - initialized with zeros.
% check the number of pixels, calculate the average value
% and store it in an array.

element = 0;	                        % number of objects in the original picture
objects = [];
average_vals=[];

for current_index=1:index
   vec= find(CLUSTER == current_index); % hold the appearances of the index in the CLUSTER      
   no_of_pixels = length(vec);		% the size of the object, in pixels	
   
   if (no_of_pixels > 20) 		% check if more than 20 pixels with the value 'element' exist in CLUSTER
      					% if there are less pixels, then it's not an object, it's 
      					% probably noise.	
      element = element + 1;
      pixel_array(element) = no_of_pixels;  % pixel_array include the no_of_pixels of each object
      object_mat = zeros(no_rows,no_cols);
      object_mat(vec)= ORIGINAL_PIC(vec);   % the relevant pixels from the original picture(before the morph. operators)      
      average_vals(element)=(sum(sum(object_mat)))/no_of_pixels; % the average value of the pixels of the object
      objects = cat(3,objects,object_mat);
     % an option to print each element 
     % figure;
     % colormap('gray');
     % imagesc(object_mat);
     % pixval on;
     % title(element);
     % pic=pic + object_mat;  
      end;
end;
   
old_average_vals = average_vals;
old_pixel_array = pixel_array;   
   
   for k=1:element
      if average_vals(k)>-100
       v=find(abs(average_vals-average_vals(k))<0.5*DIFF_PARAM);
       pixel_array(v(1))=sum(pixel_array(v));      
		for l=2:length(v)      
         objects(:,:,v(1))=objects(:,:,v(1))+objects(:,:,v(l));
      end;
      
       new_avrage=sum(sum(objects(:,:,v(1))))/pixel_array(v(1)); % calculate the new average
       average_vals(v)=-100;
       average_vals(v(1))=new_avrage;  
       
       end;
    end;
 
 old_average_vals
 average_vals
 old_pixel_array   
 pixel_array   
 
 real_objects = 0;
 for n = 1:element
     if average_vals(n)> -100
     
         real_objects = real_objects + 1;
         figure;
         colormap('gray');
         imagesc(objects(:,:,n));
         pixval on; 
         title(n);

     end;
 end;
   
 real_objects 
  
  %substruct the average value of the found objects from the original picture 
  %finding the minimum value of the picture after the substruction.if the value is negative 
  %add the absolute value of the min. to the substructed picture.
  %the substructed image will be compressed in JPEG.
  %each object will be compressed in GIF.
  gif_pic = zeros(no_rows,no_cols);
  substructed_pic=ORIGINAL_PIC;
  for m=1:element
     if average_vals(m)> -100
        obj=find(objects(:,:,m) ~= 0);
        %replace all the pixels with the average value.
        temp = objects(:,:,m);
        temp(obj)=average_vals(m);
        gif_pic = gif_pic + temp;
        objects(:,:,m)=temp ;
        
        substructed_pic=substructed_pic-objects(:,:,m);
     end;
  end;
  
%shift the values with a constant value
 substructed_pic=substructed_pic/2+127.5;
  
figure;
colormap('gray');
imagesc(gif_pic);
title('The image to be saved as gif');
pixval on;

% the picture will be saved as tif format without compression
% and will be saved as GIF with ADOBE PHOTOSHOP
imwrite(uint8(gif_pic),'new_gif.tif','Compression','none');   
  
pic2= mat2gray(substructed_pic,[0,255]);

% pic2 is the picture to be saved as JPEG.
% the picture will be compressed in different qualities
% and then will be opened and added to the GIF picture.
imwrite(pic2,'new_jpg65.jpg','jpeg','Quality',65);
gather('new_jpg65.jpg',65);
 
imwrite(pic2,'new_jpg75.jpg','jpeg','Quality',75);
gather('new_jpg75.jpg',75);
 
imwrite(pic2,'new_jpg85.jpg','jpeg','Quality',85);
gather('new_jpg85.jpg',85);
 
imwrite(pic2,'new_jpg95.jpg','jpeg','Quality',95);
gather('new_jpg95.jpg',95);
 
% the original picture will be compressed as JPEG in
% ADOBE PHOTOSHOP and will be called 'original_jpg.jpg'
% this compressing will be done before running this program.
% if the original Img is in JPEG format, this parameter is
% irrelevant.
if JPG? == 0  % the original Img is not JPEG
   compressed=imread('original_jpg.jpg');
	if Colo == 1
   	comp_gry=256*rgb2gray(im2double(compressed));       % for RGB pictures
	end;

	if Colo == 2
   	comp_gry=256*mat2gray(im2double(compressed),[0,1]); % for BW Imgs converting picture into Intencity matrice 
	end;
    mse_temp=ORIGINAL_PIC - comp_gry
 	MSE_original = mse(mse_temp)   
end;
 
if JPG? == 1
    MSE_original = 999	                                    % not relevant
end;
